@@ -1,8 +1,8 @@
 /*****************************************************************************
  * libavi.h : LibAVI library
  ******************************************************************************
- * Copyright (C) 2001-2003 VideoLAN
- * $Id: libavi.h 7512 2004-04-26 21:25:11Z gbazin $
+ * Copyright (C) 2001-2003 the VideoLAN team
+ * $Id: 453602c0754ef8ddc67f85c9bb1ada29727b59c5 $
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /* flags for use in <dwFlags> in AVIFileHdr */
@@ -224,10 +224,7 @@ int     AVI_ChunkReadRoot( stream_t *, avi_chunk_t *p_root );
 void    AVI_ChunkFreeRoot( stream_t *, avi_chunk_t  *p_chk );
 
 #define AVI_ChunkRead( s, p_chk, p_father ) \
-    _AVI_ChunkRead( s, \
-                    (avi_chunk_t*)p_chk, \
-                    (avi_chunk_t*)p_father )
-
+    _AVI_ChunkRead( s, p_chk, (avi_chunk_t*)p_father )
 #define AVI_ChunkCount( p_chk, i_fourcc ) \
     _AVI_ChunkCount( (avi_chunk_t*)p_chk, i_fourcc )
 #define AVI_ChunkFind( p_chk, i_fourcc, i_number ) \
@@ -237,14 +234,17 @@ void    AVI_ChunkFreeRoot( stream_t *, avi_chunk_t  *p_chk );
     /* *** avi stuff *** */
 
 #define AVIFOURCC_RIFF         VLC_FOURCC('R','I','F','F')
+#define AVIFOURCC_ON2          VLC_FOURCC('O','N','2',' ')
 #define AVIFOURCC_LIST         VLC_FOURCC('L','I','S','T')
 #define AVIFOURCC_JUNK         VLC_FOURCC('J','U','N','K')
 #define AVIFOURCC_AVI          VLC_FOURCC('A','V','I',' ')
 #define AVIFOURCC_AVIX         VLC_FOURCC('A','V','I','X')
+#define AVIFOURCC_ON2f         VLC_FOURCC('O','N','2','f')
 #define AVIFOURCC_WAVE         VLC_FOURCC('W','A','V','E')
 #define AVIFOURCC_INFO         VLC_FOURCC('I','N','F','O')
 
 #define AVIFOURCC_avih         VLC_FOURCC('a','v','i','h')
+#define AVIFOURCC_ON2h         VLC_FOURCC('O','N','2','h')
 #define AVIFOURCC_hdrl         VLC_FOURCC('h','d','r','l')
 #define AVIFOURCC_movi         VLC_FOURCC('m','o','v','i')
 #define AVIFOURCC_idx1         VLC_FOURCC('i','d','x','1')
@@ -259,6 +259,10 @@ void    AVI_ChunkFreeRoot( stream_t *, avi_chunk_t  *p_chk );
 #define AVIFOURCC_rec          VLC_FOURCC('r','e','c',' ')
 #define AVIFOURCC_auds         VLC_FOURCC('a','u','d','s')
 #define AVIFOURCC_vids         VLC_FOURCC('v','i','d','s')
+#define AVIFOURCC_txts         VLC_FOURCC('t','x','t','s')
+#define AVIFOURCC_mids         VLC_FOURCC('m','i','d','s')
+#define AVIFOURCC_iavs         VLC_FOURCC('i','a','v','s')
+#define AVIFOURCC_ivas         VLC_FOURCC('i','v','a','s')
 
 #define AVIFOURCC_IARL         VLC_FOURCC('I','A','R','L')
 #define AVIFOURCC_IART         VLC_FOURCC('I','A','R','T')
@@ -286,11 +290,12 @@ void    AVI_ChunkFreeRoot( stream_t *, avi_chunk_t  *p_chk );
 #define AVIFOURCC_ISMP         VLC_FOURCC('I','S','M','P')
 #define AVIFOURCC_IDIT         VLC_FOURCC('I','D','I','T')
 
-    
+ 
 #define AVITWOCC_wb            VLC_TWOCC('w','b')
 #define AVITWOCC_db            VLC_TWOCC('d','b')
 #define AVITWOCC_dc            VLC_TWOCC('d','c')
 #define AVITWOCC_pc            VLC_TWOCC('p','c')
+#define AVITWOCC_AC            VLC_TWOCC('A','C')
     /* *** codex stuff ***  */
 
     /* Microsoft RLE video */
@@ -361,3 +366,10 @@ void    AVI_ChunkFreeRoot( stream_t *, avi_chunk_t  *p_chk );
 #define FOURCC_H264         VLC_FOURCC('H','2','6','4')
 #define FOURCC_h264         VLC_FOURCC('h','2','6','4')
 #define FOURCC_VSSH         VLC_FOURCC('V','S','S','H')
+
+    /* DV */
+#define FOURCC_dvsd         VLC_FOURCC('d','v','s','d')
+#define FOURCC_dvhd         VLC_FOURCC('d','v','h','d')
+#define FOURCC_dvsl         VLC_FOURCC('d','v','s','l')
+#define FOURCC_dv25         VLC_FOURCC('d','v','2','5')
+#define FOURCC_dv50         VLC_FOURCC('d','v','5','0')
