@@ -2,7 +2,7 @@
  * cmd_playlist.cpp
  *****************************************************************************
  * Copyright (C) 2003 the VideoLAN team
- * $Id$
+ * $Id: 34f50f9ec3999d68619b7b47f8296eb987a04e4d $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulière <ipkiss@via.ecp.fr>
@@ -102,16 +102,30 @@ void CmdPlaylistSave::execute()
     if( pPlaylist != NULL )
     {
         static const char psz_xspf[] = "export-xspf",
-                          psz_m3u[] = "export-m3u";
+                          psz_m3u[] = "export-m3u",
+                          psz_html[] = "export-html";
         const char *psz_module;
         if( m_file.find( ".xsp", 0 ) != string::npos )
             psz_module = psz_xspf;
+        else if( m_file.find( "m3u", 0 ) != string::npos )
+            psz_module = psz_m3u;
+        else if( m_file.find( "html", 0 ) != string::npos )
+            psz_module = psz_html;
         else
         {
-            psz_module = psz_m3u;
-            if( m_file.find( ".m3u", 0 ) == string::npos )
-                m_file.append( ".m3u" );
+            msg_Err( getIntf(), "Impossible to recognise the file type" );
+            return;
         }
+
         playlist_Export( pPlaylist, m_file.c_str(), pPlaylist->p_local_category, psz_module );
     }
 }
+
+void CmdPlaylistFirst::execute()
+{
+    playlist_t *pPlaylist = getIntf()->p_sys->p_playlist;
+
+    playlist_Lock( pPlaylist );
+    playlist_Control( pPlaylist, PLAYLIST_PLAY, pl_Locked );
+    playlist_Unlock( pPlaylist );
+}
