@@ -1,11 +1,11 @@
 /*****************************************************************************
  * cmd_playlist.hpp
  *****************************************************************************
- * Copyright (C) 2003 VideoLAN
- * $Id: cmd_playlist.hpp 8524 2004-08-25 21:32:15Z ipkiss $
+ * Copyright (C) 2003 the VideoLAN team
+ * $Id: 4ea393ff5a90c08f9d62d422f5017c4d8d74a206 $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
- *          Olivier Teulire <ipkiss@via.ecp.fr>
+ *          Olivier Teulière <ipkiss@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #ifndef CMD_PLAYLIST_HPP
@@ -58,6 +58,9 @@ DEFINE_COMMAND( PlaylistNext, "playlist next" )
 /// Command to jump to the previous item
 DEFINE_COMMAND( PlaylistPrevious, "playlist previous" )
 
+/// Command to start the playlist
+DEFINE_COMMAND( PlaylistFirst, "playlist first" )
+
 
 /// Command to set the random state
 class CmdPlaylistRandom: public CmdGeneric
