@@ -2,7 +2,7 @@
  * var_text.hpp
  *****************************************************************************
  * Copyright (C) 2003 the VideoLAN team
- * $Id: var_text.hpp 15249 2006-04-17 15:18:05Z asmax $
+ * $Id: 0c4e34aa9ca02466419e828cbd2a65788c5cf9bd $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulière <ipkiss@via.ecp.fr>
@@ -32,9 +32,9 @@
 
 
 /// String variable
-class VarText: public Variable, public Subject<VarText, void*>,
-               public Observer<VarPercent, void*>,
-               public Observer< VarText,void*>
+class VarText: public Variable, public Subject<VarText>,
+               public Observer<VarPercent>,
+               public Observer<VarText>
 {
     public:
         // Set substVars to true to replace "$X" variables in the text
@@ -49,8 +49,8 @@ class VarText: public Variable, public Subject<VarText, void*>,
         virtual const UString get() const;
 
         /// Methods called when an observed variable is modified
-        virtual void onUpdate( Subject<VarPercent, void*> &rVariable, void* );
-        virtual void onUpdate( Subject<VarText, void*> &rVariable, void* );
+        virtual void onUpdate( Subject<VarPercent> &rVariable, void* );
+        virtual void onUpdate( Subject<VarText> &rVariable, void* );
 
     private:
         /// Stop observing other variables
