@@ -2,7 +2,7 @@
  * intf.c : audio output API towards the interface modules
  *****************************************************************************
  * Copyright (C) 2002-2007 the VideoLAN team
- * $Id: 3e652f3caa19c4377123d827d043835766035d1a $
+ * $Id: a780db6b251674f3bad957e405cf6c4b9ce5aad6 $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -98,14 +98,11 @@ int __aout_VolumeGet( vlc_object_t * p_object, audio_volume_t * pi_volume )
  *****************************************************************************/
 int __aout_VolumeSet( vlc_object_t * p_object, audio_volume_t i_volume )
 {
-    vlc_value_t val;
     aout_instance_t *p_aout = vlc_object_find( p_object, VLC_OBJECT_AOUT, FIND_ANYWHERE );
     int i_result = 0;
 
     config_PutInt( p_object, "volume", i_volume );
-
-    val.b_bool = true;
-    var_Set( p_object->p_libvlc, "volume-change", val );
+    var_SetBool( p_object->p_libvlc, "volume-change", true );
 
     if ( p_aout == NULL ) return 0;
 
@@ -116,7 +113,7 @@ int __aout_VolumeSet( vlc_object_t * p_object, audio_volume_t i_volume )
     }
     aout_unlock_mixer( p_aout );
 
-    var_Set( p_aout, "intf-change", val );
+    var_SetBool( p_aout, "intf-change", true );
     vlc_object_release( p_aout );
     return i_result;
 }
@@ -157,7 +154,6 @@ int __aout_VolumeInfos( vlc_object_t * p_object, audio_volume_t * pi_soft )
 int __aout_VolumeUp( vlc_object_t * p_object, int i_nb_steps,
                    audio_volume_t * pi_volume )
 {
-    vlc_value_t val;
     aout_instance_t * p_aout = vlc_object_find( p_object, VLC_OBJECT_AOUT,
                                                 FIND_ANYWHERE );
     int i_result = 0, i_volume = 0, i_volume_step = 0;
@@ -175,8 +171,7 @@ int __aout_VolumeUp( vlc_object_t * p_object, int i_nb_steps,
                     (audio_volume_t) i_volume );
     if ( pi_volume != NULL ) *pi_volume = (audio_volume_t) i_volume;
 
-    val.b_bool = true;
-    var_Set( p_object->p_libvlc, "volume-change", val );
+    var_SetBool( p_object->p_libvlc, "volume-change", true );
 
     if ( p_aout == NULL ) return 0;
 
@@ -201,7 +196,6 @@ int __aout_VolumeUp( vlc_object_t * p_object, int i_nb_steps,
 int __aout_VolumeDown( vlc_object_t * p_object, int i_nb_steps,
                      audio_volume_t * pi_volume )
 {
-    vlc_value_t val;
     aout_instance_t * p_aout = vlc_object_find( p_object, VLC_OBJECT_AOUT,
                                                 FIND_ANYWHERE );
     int i_result = 0, i_volume = 0, i_volume_step = 0;
@@ -218,8 +212,7 @@ int __aout_VolumeDown( vlc_object_t * p_object, int i_nb_steps,
     var_SetInteger( p_object->p_libvlc, "saved-volume", (audio_volume_t) i_volume );
     if ( pi_volume != NULL ) *pi_volume = (audio_volume_t) i_volume;
 
-    val.b_bool = true;
-    var_Set( p_object->p_libvlc, "volume-change", val );
+    var_SetBool( p_object->p_libvlc, "volume-change", true );
 
     if ( p_aout == NULL ) return 0;
 
@@ -420,7 +413,7 @@ static int aout_Restart( aout_instance_t * p_aout )
     for ( i = 0; i < p_aout->i_nb_inputs; i++ )
     {
         aout_input_t * p_input = p_aout->pp_inputs[i];
-        b_error |= aout_InputNew( p_aout, p_input );
+        b_error |= aout_InputNew( p_aout, p_input, &p_input->request_vout );
         p_input->b_changed = 1;
         aout_unlock_input( p_aout, p_input );
     }
