@@ -2,7 +2,7 @@
  * mixer.c : audio output mixing operations
  *****************************************************************************
  * Copyright (C) 2002-2004 the VideoLAN team
- * $Id: 45255c7afdcf4a759f40091a6af8aaca18b16c6d $
+ * $Id: 4f1f9b64899a5d0c2989ab2d2a2b14ba09e6d83b $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -43,7 +43,7 @@
  *****************************************************************************/
 int aout_MixerNew( aout_instance_t * p_aout )
 {
-    p_aout->mixer.p_module = module_Need( p_aout, "audio mixer", NULL, 0 );
+    p_aout->mixer.p_module = module_need( p_aout, "audio mixer", NULL, false );
     if ( p_aout->mixer.p_module == NULL )
     {
         msg_Err( p_aout, "no suitable audio mixer" );
@@ -61,7 +61,7 @@ int aout_MixerNew( aout_instance_t * p_aout )
 void aout_MixerDelete( aout_instance_t * p_aout )
 {
     if ( p_aout->mixer.b_error ) return;
-    module_Unneed( p_aout, p_aout->mixer.p_module );
+    module_unneed( p_aout, p_aout->mixer.p_module );
     p_aout->mixer.b_error = 1;
 }
 
@@ -131,7 +131,8 @@ static int MixBuffer( aout_instance_t * p_aout )
             aout_fifo_t * p_fifo = &p_input->fifo;
             aout_buffer_t * p_buffer;
 
-            if ( p_input->b_error ) continue;
+            if ( p_input->b_error || p_input->b_paused )
+                continue;
 
             p_buffer = p_fifo->p_first;
             while ( p_buffer != NULL && p_buffer->start_date < mdate() )
@@ -140,11 +141,6 @@ static int MixBuffer( aout_instance_t * p_aout )
                           "trashing", mdate() - p_buffer->start_date );
                 p_buffer = aout_FifoPop( p_aout, p_fifo );
                 aout_BufferFree( p_buffer );
-                if( p_input->p_input_thread )
-                {
-//                    stats_UpdateInteger( p_input->p_input_thread,
-//                                         "lost_abuffers", 1 );
-                }
                 p_buffer = p_fifo->p_first;
                 p_input->p_first_byte_to_mix = NULL;
             }
@@ -181,7 +177,7 @@ static int MixBuffer( aout_instance_t * p_aout )
         mtime_t prev_date;
         bool b_drop_buffers;
 
-        if ( p_input->b_error )
+        if ( p_input->b_error || p_input->b_paused )
         {
             if ( i_first_input == i ) i_first_input++;
             continue;
@@ -202,11 +198,6 @@ static int MixBuffer( aout_instance_t * p_aout )
             msg_Warn( p_aout, "the mixer got a packet in the past (%"PRId64")",
                       start_date - p_buffer->end_date );
             aout_BufferFree( p_buffer );
-            if( p_input->p_input_thread )
-            {
-//                stats_UpdateInteger( p_input->p_input_thread,
-//                                     "lost_abuffers", 1 );
-            }
             p_fifo->p_first = p_buffer = p_next;
             p_input->p_first_byte_to_mix = NULL;
         }
